import django_tables2 as tables
from django_tables2.utils import Accessor

from netbox.tables import NetBoxTable, columns
from django.db.models import Count

from . import models
from . import columns as riskColumns

# ThreatSource Tables


class ThreatSourceTable(NetBoxTable):

    name = tables.Column(linkify=True)

    class Meta(NetBoxTable.Meta):
        model = models.ThreatSource
        fields = ["name", "threat_type", "capability", "intent", "targeting"]


# ThreatEvent Tables


class ThreatEventTable(NetBoxTable):

    name = tables.Column(linkify=True)

    class Meta(NetBoxTable.Meta):
        model = models.ThreatEvent
        fields = ["name", "threat_source", "relevance", "likelihood", "impact"]


# Vulnerability Tables


class VulnerabilityTable(NetBoxTable):

    name = tables.Column(linkify=True)
    affected = columns.LinkedCountColumn(
        verbose_name="Affected Assets",
        accessor=Accessor("affected_assets"),
        viewname='plugins:nb_risk:vulnerabilityassignment_list',
        url_params={
            'vulnerability': 'name',
        },
    )

    def order_affected(self, queryset, is_descending):
        if is_descending:
            queryset = queryset.annotate(affected_assets=Count('vulnerability_assignments')).order_by('-affected_assets')
        else:
            queryset = queryset.annotate(affected_assets=Count('vulnerability_assignments')).order_by('affected_assets')
        return (queryset, True)

    class Meta(NetBoxTable.Meta):
        model = models.Vulnerability
        fields = ["name", "cve", "description", "affected", "cvssbaseScore"]


# VulnerabilityAssignment Tables


class VulnerabilityAssignmentTable(NetBoxTable):

    actions = columns.ActionsColumn(actions=("delete",))
    vulnerability = tables.Column(linkify=True)

    class Meta(NetBoxTable.Meta):
        model = models.VulnerabilityAssignment
        fields = ["vulnerability", "vulnerability__cve", "vulnerability__description"]


class VulnerabilityAssignmentListTable(NetBoxTable):

    actions = columns.ActionsColumn(actions=("delete",))

    asset = tables.Column(linkify=True)

    asset_object_type = tables.Column(verbose_name="Asset Type")

    class Meta(NetBoxTable.Meta):
        model = models.VulnerabilityAssignment
        fields = ["asset", "asset_object_type"]


class VulnerabilityExploitListTable(NetBoxTable):

    actions = columns.ActionsColumn(actions=("delete",))

    asset = tables.Column(linkify=True)

    vulnerability = tables.Column(linkify=True)

    asset_object_type = tables.Column(verbose_name="Asset Type")

    class Meta(NetBoxTable.Meta):
        model = models.VulnerabilityAssignment
        fields = ["asset", "vulnerability", "asset_object_type"]


# Risk Tables


class RiskTable(NetBoxTable):

    name = tables.Column(linkify=True)

    threat_event = tables.Column(linkify=True)

    risk_level = tables.Column(verbose_name="Risk Level")

    class Meta(NetBoxTable.Meta):
        model = models.Risk
        fields = ["name", "threat_event", "likelihood", "impact", "risk_level"]


# CVE Tables


class CveTable(tables.Table):

    id = tables.Column(attrs={"td": {"class": "text-end text-nowrap"}})
    description = tables.Column()
    accessVector = tables.Column(verbose_name="Access Vector")
    accessComplexity = tables.Column(verbose_name="Access Complexity")
    authentication = tables.Column(verbose_name="Authentication")
    confidentialityImpact = tables.Column(verbose_name="Confidentiality Impact")
    integrityImpact = tables.Column(verbose_name="Integrity Impact")
    availabilityImpact = tables.Column(verbose_name="Availability Impact")
    baseScore = tables.Column(verbose_name="Base Score")

    create = riskColumns.CreateColumn(empty_values=())

    class Meta:
        attrs = {
            "class": "table table-hover object-list",
        }

# Control Tables

class ControlTable(NetBoxTable):
    name = tables.Column(linkify=True)
    class Meta(NetBoxTable.Meta):
        model = models.Control
        fields = ["name",]



# Generated by Django 4.1.9 on 2023-05-13 16:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('nb_risk', '0004_alter_vulnerability_options'),
    ]

    operations = [
        migrations.AlterField(
            model_name='vulnerability',
            name='cvssbaseScore',
            field=models.FloatField(blank=True, max_length=100, null=True),
        ),
    ]



# Generated by Django 4.1.5 on 2023-05-10 13:39

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('nb_risk', '0003_control'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='vulnerability',
            options={'verbose_name': 'Vulnerability', 'verbose_name_plural': 'Vulnerabilities'},
        ),
    ]



# Generated by Django 4.1.5 on 2023-04-21 01:36

from django.db import migrations, models
import taggit.managers
import utilities.json


class Migration(migrations.Migration):

    dependencies = [
        ('extras', '0084_staging'),
        ('nb_risk', '0002_vulnerability_cvssaccesscomplexity_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='Control',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('name', models.CharField(max_length=100, unique=True)),
                ('description', models.CharField(blank=True, max_length=100)),
                ('notes', models.TextField(blank=True)),
                ('category', models.CharField(default='Preventive', max_length=100)),
                ('risk', models.ManyToManyField(blank=True, to='nb_risk.risk')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from django import forms

from netbox.forms import NetBoxModelForm, NetBoxModelFilterSetForm, NetBoxModelImportForm
from dcim.models import Device, DeviceType
from utilities.forms import BootstrapMixin
from utilities.forms.fields import (
    DynamicModelMultipleChoiceField,
    SlugField,
    DynamicModelChoiceField,
)

from . import models, choices

# ThreatSource Forms


class ThreatSourceForm(NetBoxModelForm):
    class Meta:
        model = models.ThreatSource
        fields = [
            "name",
            "threat_type",
            "capability",
            "intent",
            "targeting",
            "description",
            "notes",
        ]


class ThreatSourceFilterForm(NetBoxModelFilterSetForm):
    model = models.ThreatSource

    class Meta:
        fields = ["name", "threat_type", "capability", "intent", "targeting"]


# ThreatEvent Forms


class ThreatEventForm(NetBoxModelForm):

    vulnerability = DynamicModelMultipleChoiceField(
        queryset=models.VulnerabilityAssignment.objects.all(),
        required=True,
    )

    class Meta:
        model = models.ThreatEvent
        fields = [
            "name",
            "threat_source",
            "description",
            "notes",
            "relevance",
            "likelihood",
            "impact",
            "vulnerability",
        ]


class ThreatEventFilterForm(NetBoxModelFilterSetForm):
    model = models.ThreatEvent

    class Meta:
        fields = [
            "name",
            "threat_source",
            "relevance",
            "likelihood",
            "impact",
            "vulnerability",
        ]


# Vulnerability Forms


class VulnerabilityForm(NetBoxModelForm):

    fieldsets = (
        ("Vulnerability", ("name", "cve", "description", "notes"),),
        ("CVSSv2 Score", ("cvssaccessVector", "cvssaccessComplexity", "cvssauthentication", "cvssconfidentialityImpact", "cvssintegrityImpact", "cvssavailabilityImpact", "cvssbaseScore")),
    )
    class Meta:
        model = models.Vulnerability
        fields = [
            "name",
            "cve",
            "description",
            "notes",
            "cvssaccessVector",
            "cvssaccessComplexity",
            "cvssauthentication",
            "cvssconfidentialityImpact",
            "cvssintegrityImpact",
            "cvssavailabilityImpact",
            "cvssbaseScore",
        ]


class VulnerabilityFilterForm(NetBoxModelFilterSetForm):
    model = models.Vulnerability

    class Meta:
        fields = ["name", "cve"]


class VulnerabilitySearchFilterForm(NetBoxModelFilterSetForm):
    model = models.Vulnerability
    q = forms.CharField(label="cpeName", required=False)

    device_type = DynamicModelChoiceField(
        queryset=DeviceType.objects.all(),
        required=False,
    )
    version = forms.CharField(required=False)

    part = forms.ChoiceField(
        choices=choices.CVE_PART_CHOICES,
        required=False,
    )

class VulnerabilityImportForm(NetBoxModelImportForm):
    class Meta:
        model = models.Vulnerability
        fields = [
            "name",
            "cve",
            "description",
            "notes",
            "cvssaccessVector",
            "cvssaccessComplexity",
            "cvssauthentication",
            "cvssconfidentialityImpact",
            "cvssintegrityImpact",
            "cvssavailabilityImpact",
            "cvssbaseScore",
        ]

# VulnerabilityAssignment Forms


class VulnerabilityAssignmentForm(BootstrapMixin, forms.ModelForm):

    vulnerability = DynamicModelChoiceField(
        queryset=models.Vulnerability.objects.all(),
        required=True,
    )

    class Meta:
        model = models.VulnerabilityAssignment
        fields = ["asset_object_type", "asset_id", "vulnerability"]
        widgets = {
            "asset_object_type": forms.HiddenInput(),
            "asset_id": forms.HiddenInput(),
        }


class VulnerabilityAssignmentFilterForm(NetBoxModelFilterSetForm):
    model = models.VulnerabilityAssignment

    class Meta:
        fields = ["vulnerability"]


# Risk Forms


class RiskForm(NetBoxModelForm):
    class Meta:
        model = models.Risk
        fields = [
            "name",
            "threat_event",
            "description",
            "notes",
            "likelihood",
            "impact",
        ]


class RiskFilterForm(NetBoxModelFilterSetForm):
    model = models.Risk

    class Meta:
        fields = ["name", "threat_event", "description", "impact", "likelihood"]

# Control Forms

class ControlForm(NetBoxModelForm):
    class Meta:
        model = models.Control
        fields = [
            "name",
            "description",
            "notes",
            "category",
            "risk",
        ]

class ControlFilterForm(NetBoxModelFilterSetForm):
    model = models.Control

    class Meta:
        fields = [
            "name",
            "description",
            "notes",
            "category",
            "risk",
        ]


# Generated by Django 4.1.10 on 2023-08-20 02:18

from django.db import migrations, models
import django.db.models.functions.text


class Migration(migrations.Migration):

    dependencies = [
        ('nb_risk', '0006_vulnerability_unique_vuln_name'),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name='vulnerability',
            name='unique_vuln_name',
        ),
        migrations.AddConstraint(
            model_name='vulnerability',
            constraint=models.UniqueConstraint(django.db.models.functions.text.Lower('name'), name='unique_vuln_name'),
        ),
    ]



from netbox.views import generic
from dcim.models import Device, Site
from tenancy.models import Tenant
from virtualization.models import VirtualMachine

from extras.plugins.utils import get_plugin_config
from utilities.views import ViewTab, register_model_view
from django.contrib.contenttypes.models import ContentType
from django.shortcuts import get_object_or_404

import logging

logger = logging.getLogger(__name__)

from . import forms, models, tables, filters

# ThreatSource Views


class ThreatSourceView(generic.ObjectView):
    queryset = models.ThreatSource.objects.all()


class ThreatSourceListView(generic.ObjectListView):
    queryset = models.ThreatSource.objects.all()
    table = tables.ThreatSourceTable
    filterset = filters.ThreatSourceFilterSet
    filterset_form = forms.ThreatSourceFilterForm


class ThreatSourceEditView(generic.ObjectEditView):
    queryset = models.ThreatSource.objects.all()
    form = forms.ThreatSourceForm


class ThreatSourceDeleteView(generic.ObjectDeleteView):
    queryset = models.ThreatSource.objects.all()


# ThreatEvent Views


class ThreatEventView(generic.ObjectView):
    queryset = models.ThreatEvent.objects.all()


@register_model_view(models.ThreatEvent, name='vulnerabilities')
class ThreatEventVulnerabilityView(generic.ObjectChildrenView):
    queryset = models.ThreatEvent.objects.all()
    child_model = models.Vulnerability
    table = tables.VulnerabilityExploitListTable
    template_name = "nb_risk/threatevent_vulnerabilities.html"
    tab = ViewTab(label='Exploit Vulnerabilities', badge=lambda obj: obj.vulnerability.all().count(), hide_if_empty=True)

    def get_children(self, request, parent):
            childrens = parent.vulnerability.all()
            return childrens


class ThreatEventListView(generic.ObjectListView):
    queryset = models.ThreatEvent.objects.all()
    table = tables.ThreatEventTable
    filterset = filters.ThreatEventFilterSet
    filterset_form = forms.ThreatEventFilterForm


class ThreatEventEditView(generic.ObjectEditView):
    queryset = models.ThreatEvent.objects.all()
    form = forms.ThreatEventForm


class ThreatEventDeleteView(generic.ObjectDeleteView):
    queryset = models.ThreatEvent.objects.all()


class ThreatEventBulkDeleteView(generic.BulkDeleteView):
    queryset = models.ThreatEvent.objects.all()
    table = tables.ThreatEventTable


# Vulnerability Views


class VulnerabilityView(generic.ObjectView):
    queryset = models.Vulnerability.objects.all()


@register_model_view(models.Vulnerability, name='affected_assets')
class VulnerabilityAffectedAssetsView(generic.ObjectChildrenView):
    queryset = models.Vulnerability.objects.all()
    child_model = models.Vulnerability
    table = tables.VulnerabilityAssignmentListTable
    template_name = "nb_risk/vulnerability_affected_assets.html"
    tab = ViewTab(label='Affected Assets', badge=lambda obj: models.VulnerabilityAssignment.objects.filter(vulnerability=obj).count(), hide_if_empty=True)

    def get_children(self, request, parent):
            childrens = models.VulnerabilityAssignment.objects.filter(vulnerability=parent)
            return childrens

    
class VulnerabilityListView(generic.ObjectListView):
    queryset = models.Vulnerability.objects.all()
    table = tables.VulnerabilityTable
    filterset = filters.VulnerabilityFilterSet
    filterset_form = forms.VulnerabilityFilterForm
    template_name = "nb_risk/vulnerability_list.html"


class VulnerabilityEditView(generic.ObjectEditView):
    queryset = models.Vulnerability.objects.all()
    form = forms.VulnerabilityForm

class VulnerabilityBulkImportView(generic.BulkImportView):
    queryset = models.Vulnerability.objects.all()
    model_form = forms.VulnerabilityImportForm

class VulnerabilityDeleteView(generic.ObjectDeleteView):
    queryset = models.Vulnerability.objects.all()


class VulnerabilityBulkDeleteView(generic.BulkDeleteView):
    queryset = models.Vulnerability.objects.all()
    table = tables.VulnerabilityTable


# Generic Vulnerability List View


def create_view(model_representation):

    app_label, model = model_representation.split(".")
    model = ContentType.objects.get(app_label=app_label, model=model).model_class()
    name = f"{model._meta.model_name}-vulnerability_list"
    path = f"{model._meta.model_name}-vulnerability_list"
    base_template_name = f"{model._meta.app_label}/{model._meta.model_name}.html"

    class View(generic.ObjectChildrenView):
        def __init__(self, *args, **kwargs):
            self.queryset = model.objects.all()
            self.child_model = models.VulnerabilityAssignment
            self.table = tables.VulnerabilityAssignmentTable
            self.template_name = "nb_risk/generic_vulnerability_list.html"
            super().__init__(*args, **kwargs)

        tab = ViewTab(
            label="Vulnerabilities",
            badge=lambda obj: models.VulnerabilityAssignment.objects.filter(
                asset_object_type=ContentType.objects.get_for_model(obj),
                asset_id=obj.id,
            ).count(),
            hide_if_empty=True,
        )

        def get_children(self, request, parent):
            childrens = self.child_model.objects.filter(
                asset_object_type=ContentType.objects.get_for_model(parent),
                asset_id=parent.id,
            )
            return childrens

        def get_extra_context(self, request, instance):
            data = {
                "base_template_name": base_template_name,
            }
            return data

    register_model_view(model, name=name, path=path)(View)


supported_assets = get_plugin_config("nb_risk", "supported_assets")
additional_assets = get_plugin_config("nb_risk", "additional_assets")
supported_assets = supported_assets + additional_assets

for supported_asset in supported_assets:
    create_view(supported_asset)

# VulnerabilityAssignment Views


class VulnerabilityAssignmentEditView(generic.ObjectEditView):
    queryset = models.VulnerabilityAssignment.objects.all()
    form = forms.VulnerabilityAssignmentForm
    template_name = "nb_risk/generic_vulnerability_assignment_edit.html"

    def alter_object(self, instance, request, args, kwargs):
        if not instance.pk:
            # Assign the object based on URL kwargs
            content_type = get_object_or_404(
                ContentType, pk=request.GET.get("asset_object_type")
            )
            instance.object = get_object_or_404(
                content_type.model_class(), pk=request.GET.get("asset_id")
            )
        return instance

    def get_extra_addanother_params(self, request):
        return {
            "asset_object_type": request.GET.get("asset_object_type"),
            "asset_id": request.GET.get("asset_id"),
        }


class VulnerabilityAssignmentDeleteView(generic.ObjectDeleteView):
    queryset = models.VulnerabilityAssignment.objects.all()

class VulnerabilityAssignmentListView(generic.ObjectListView):
    queryset = models.VulnerabilityAssignment.objects.all()
    table = tables.VulnerabilityExploitListTable
    filterset = filters.VulnerabilityAssignmentFilterSet
    filterset_form = forms.VulnerabilityAssignmentFilterForm
    actions = ('import', 'export', )

# Risk Views


class RiskListView(generic.ObjectListView):
    queryset = models.Risk.objects.all()
    table = tables.RiskTable
    filterset = filters.RiskFilterSet
    filterset_form = forms.RiskFilterForm


class RiskView(generic.ObjectView):
    queryset = models.Risk.objects.all()


class RiskEditView(generic.ObjectEditView):
    queryset = models.Risk.objects.all()
    form = forms.RiskForm


class RiskDeleteView(generic.ObjectDeleteView):
    queryset = models.Risk.objects.all()


class RiskBulkDeleteView(generic.BulkDeleteView):
    queryset = models.Risk.objects.all()
    table = tables.RiskTable

# Control Views

class ControlListView(generic.ObjectListView):
    queryset = models.Control.objects.all()
    table = tables.ControlTable
    filterset = filters.ControlFilterSet
    filterset_form = forms.ControlFilterForm

class ControlView(generic.ObjectView):
    queryset = models.Control.objects.all()

    def get_extra_context(self, request, instance):
        risks = instance.risk.all()
        thread_events = []
        for risk in risks:
            thread_events.append(risk.threat_event)
        assets = models.VulnerabilityAssignment.objects.filter(threat_events__in=thread_events)
        data = {
            "risks_count": risks.count(),
            "assets_count": assets.count(),
        }
        return data

@register_model_view(models.Control, name='risks')
class ControlRisksView(generic.ObjectChildrenView):
    queryset = models.Control.objects.all()
    child_model = models.Risk
    table = tables.RiskTable
    template_name = "nb_risk/control_risks.html"
    tab = ViewTab(label='Risks', badge=lambda obj: obj.risk.all().count(), hide_if_empty=True)

    def get_children(self, request, parent):
            childrens = parent.risk.all()
            return childrens

def _get_assets(instance):
        risks = instance.risk.all()
        thread_events = []
        for risk in risks:
            thread_events.append(risk.threat_event)
        assets = models.VulnerabilityAssignment.objects.filter(threat_events__in=thread_events)
        return assets

@register_model_view(models.Control, name='assets')
class ControlAssetsView(generic.ObjectChildrenView):
    queryset = models.Control.objects.all()
    child_model = models.VulnerabilityAssignment
    table = tables.VulnerabilityAssignmentListTable
    template_name = "nb_risk/control_assets.html"
    tab = ViewTab(label='Related Assets', badge=lambda obj: _get_assets(obj).count(), hide_if_empty=True)


    def get_children(self, request, parent):
            childrens = _get_assets(parent)
            return childrens

class ControlEditView(generic.ObjectEditView):
    queryset = models.Control.objects.all()
    form = forms.ControlForm

class ControlDeleteView(generic.ObjectDeleteView):
    queryset = models.Control.objects.all()

class ControlBulkDeleteView(generic.BulkDeleteView):
    queryset = models.Control.objects.all()
    table = tables.ControlTable



from netbox.api.routers import NetBoxRouter
from . import views

app_name = 'nb_risk'

router = NetBoxRouter()

router.register('threatsource', views.ThreatSourceViewSet)
router.register('threatevent', views.ThreatEventViewSet)
router.register('vulnerability', views.VulnerabilityViewSet)
router.register('vulnerabilityassignment', views.VulnerabilityAssignmentViewSet)
router.register('risk', views.RiskViewSet)
router.register('control', views.ControlViewSet)


urlpatterns = router.urls


from rest_framework import serializers
from django.contrib.contenttypes.models import ContentType
from netbox.api.fields import ChoiceField, ContentTypeField, SerializedPKRelatedField
from netbox.api.serializers import WritableNestedSerializer
from utilities.api import get_serializer_for_model


from netbox.api.serializers import NetBoxModelSerializer
from nb_risk.api.nested_serializers import (
    NestedThreatSourceSerializer,
    NestedRiskSerializer,
)
from .. import models, choices

# ThreatSource Serializers

class ThreatSourceSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name="plugins-api:nb_risk-api:threatsource-detail")
    display = serializers.SerializerMethodField('get_display')
    threat_type = ChoiceField(choices=choices.ThreatTypeChoices)
    capability = ChoiceField(choices=choices.CapabilityChoices)

    def get_display(self, obj):
        return obj.name

    class Meta:
        model = models.ThreatSource
        fields = [
            "id",
            "url",
            "display",
            "name",
            "threat_type",
            "capability",
            "intent",
            "targeting",
            "description",
        ]

# ThreatEvent Serializers

class ThreatEventSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name="plugins-api:nb_risk-api:threatevent-detail")
    display = serializers.SerializerMethodField('get_display')
    threat_source = serializers.SlugRelatedField(slug_field="name", queryset=models.ThreatSource.objects.all())
    relevance = ChoiceField(choices=choices.RelevanceChoices)
    likelihood = ChoiceField(choices=choices.LikelihoodChoices)


    def get_display(self, obj):
        return obj.name

    class Meta:
        model = models.ThreatEvent
        fields = [
            "id",
            "url",
            "display",
            "name",
            "threat_source",
            "relevance",
            "likelihood",
            "impact",
            "vulnerability",
        ]

# Vulnerability Serializers

class VulnerabilitySerializer(NetBoxModelSerializer):
    
    url = serializers.HyperlinkedIdentityField(view_name="plugins-api:nb_risk-api:vulnerability-detail")
    display = serializers.SerializerMethodField('get_display')

    def get_display(self, obj):
        return obj.name

    class Meta:
        model = models.Vulnerability
        fields = [
            "id",
            "url",
            "display",
            "name",
            "cve",
            "description",
        ]


# VulnerabilityAssignment Serializers

class VulnerabilityAssignmentSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name="plugins-api:nb_risk-api:vulnerabilityassignment-detail")
    display = serializers.SerializerMethodField('get_display')
    asset_object_type = ContentTypeField(
        queryset=ContentType.objects.filter(choices.AssetTypes),
        required=True,
        allow_null=True
    )
    asset = serializers.SerializerMethodField(read_only=True)
    vulnerability = serializers.SlugRelatedField(slug_field="name", queryset=models.Vulnerability.objects.all())

    asset_object_id = serializers.IntegerField(source='asset.id')

    def get_asset(self, obj):
        if obj.asset is None:
            return None
        serializer = get_serializer_for_model(obj.asset, prefix='Nested')
        context = {'request': self.context['request']}
        return serializer(obj.asset, context=context).data

    def get_display(self, obj):
        return obj.name

    class Meta:
        model = models.VulnerabilityAssignment
        fields = [
            "id",
            "url",
            "display",
            "asset_object_type",
            "asset_object_id",
            "asset",
            "vulnerability",
        ]

# Risk Serializers

class RiskSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name="plugins-api:nb_risk-api:risk-detail")
    display = serializers.SerializerMethodField('get_display')
    
    threat_event = serializers.SlugRelatedField(slug_field="name", queryset=models.ThreatEvent.objects.all())

   
    def get_display(self, obj):
        return obj.name

    class Meta:
        model = models.Risk
        fields = [
            "id",
            "url",
            "display",
            "threat_event",
            "description",
            "likelihood",
            "impact",
            "notes",
        ]

# Control Serializers

class ControlSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name="plugins-api:nb_risk-api:control-detail")
    display = serializers.SerializerMethodField('get_display')
    risk = NestedRiskSerializer(many=True,required=False, allow_null=True)

    def get_display(self, obj):
        return obj.name
    
    class Meta:
        model = models.Control
        fields = [
            "id",
            "url",
            "display",
            "name",
            "description",
            "notes",
            "risk"
        ]


from django.db.models import Q
from utilities.choices import ChoiceSet

# Define a set of choices for the Threat Type
class ThreatTypeChoices(ChoiceSet):

    THREAT_TYPE_1 = "ADVERSARIAL"
    THREAT_TYPE_2 = "ACCIDENTAL"
    THREAT_TYPE_3 = "STRUCTURAL"
    THREAT_TYPE_4 = "ENVIRONMENTAL"

    CHOICES = (
        (THREAT_TYPE_1, "ADVERSARIAL"),
        (THREAT_TYPE_2, "ACCIDENTAL"),
        (THREAT_TYPE_3, "STRUCTURAL"),
        (THREAT_TYPE_4, "ENVIRONMENTAL"),
    )


# Define a set of choices for the Capability
class CapabilityChoices(ChoiceSet):

    CAPABILITY_1 = "Very High"
    CAPABILITY_2 = "High"
    CAPABILITY_3 = "Moderate"
    CAPABILITY_4 = "Low"
    CAPABILITY_5 = "Very Low"

    CHOICES = (
        (CAPABILITY_1, "Very High"),
        (CAPABILITY_2, "High"),
        (CAPABILITY_3, "Moderate"),
        (CAPABILITY_4, "Low"),
        (CAPABILITY_5, "Very Low"),
    )


# Define a set of choices for the Relevance


class RelevanceChoices(ChoiceSet):

    RELEVANCE_1 = "Confirmed"
    RELEVANCE_2 = "Expected"
    RELEVANCE_3 = "Anticipated"
    RELEVANCE_4 = "Predicted"
    RELEVANCE_5 = "Possible"
    RELEVANCE_6 = "N/A"

    CHOICES = (
        (RELEVANCE_1, "Confirmed"),
        (RELEVANCE_2, "Expected"),
        (RELEVANCE_3, "Anticipated"),
        (RELEVANCE_4, "Predicted"),
        (RELEVANCE_5, "Possible"),
        (RELEVANCE_6, "N/A"),
    )


# Define a set of choices for the Likelihood


class LikelihoodChoices(ChoiceSet):

    LIKELIHOOD_1 = "Very High"
    LIKELIHOOD_2 = "High"
    LIKELIHOOD_3 = "Moderate"
    LIKELIHOOD_4 = "Low"
    LIKELIHOOD_5 = "Very Low"

    CHOICES = (
        (LIKELIHOOD_1, "Very High"),
        (LIKELIHOOD_2, "High"),
        (LIKELIHOOD_3, "Moderate"),
        (LIKELIHOOD_4, "Low"),
        (LIKELIHOOD_5, "Very Low"),
    )


# Define a set of choices for the Impact


class ImpactChoices(ChoiceSet):

    IMPACT_1 = "Very High"
    IMPACT_2 = "High"
    IMPACT_3 = "Moderate"
    IMPACT_4 = "Low"
    IMPACT_5 = "Very Low"

    CHOICES = (
        (IMPACT_1, "Very High"),
        (IMPACT_2, "High"),
        (IMPACT_3, "Moderate"),
        (IMPACT_4, "Low"),
        (IMPACT_5, "Very Low"),
    )


# Define Asset Type Choices

AssetTypes = Q(
    Q(app_label="dcim", model="device")
    | Q(app_label="virtualization", model="virtualmachine")
)

# Define Level of Risk Choices


class RiskLevelChoices(ChoiceSet):

    RISK_LEVEL_1 = "Very High"
    RISK_LEVEL_2 = "High"
    RISK_LEVEL_3 = "Moderate"
    RISK_LEVEL_4 = "Low"
    RISK_LEVEL_5 = "Very Low"

    CHOICES = (
        (RISK_LEVEL_1, "Very High"),
        (RISK_LEVEL_2, "High"),
        (RISK_LEVEL_3, "Moderate"),
        (RISK_LEVEL_4, "Low"),
        (RISK_LEVEL_5, "Very Low"),
    )


# Define CVE part choices


class CVE_PART_CHOICES(ChoiceSet):

    PART_1 = "a"
    PART_2 = "o"
    PART_3 = "h"

    CHOICES = (
        (PART_1, "Applications"),
        (PART_2, "Operating Systems"),
        (PART_3, "Hardware Devices"),
    )

# ControlCategoryChoices

class ControlCategoryChoices(ChoiceSet):

    CATEGORY_1 = "Preventive"
    CATEGORY_2 = "Detective"

    CHOICES = (
        (CATEGORY_1, "Preventive"),
        (CATEGORY_2, "Detective"),
    )



# Generated by Django 4.1.9 on 2023-05-13 17:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('nb_risk', '0005_alter_vulnerability_cvssbasescore'),
    ]

    operations = [
        migrations.AddConstraint(
            model_name='vulnerability',
            constraint=models.UniqueConstraint(fields=('name',), name='unique_vuln_name'),
        ),
    ]



from dcim.models import Device, Site, DeviceType
from tenancy.models import Tenant
from virtualization.models import VirtualMachine

from utilities.views import ViewTab, register_model_view
from django.contrib.contenttypes.models import ContentType
from django.shortcuts import get_object_or_404

from django.views.generic import View
from django.shortcuts import redirect, render

from utilities.views import ObjectPermissionRequiredMixin
from utilities.permissions import get_permission_for_model

import logging
import requests

logger = logging.getLogger(__name__)

from . import forms, models, tables, filters

# Vulnerability Search Views


class VulnerabilitySearchView(ObjectPermissionRequiredMixin, View):
    queryset = models.Vulnerability.objects.all()
    template_name = "nb_risk/vulnerability_search.html"
    tab = None
    filterset_form = forms.VulnerabilitySearchFilterForm

    def get_required_permission(self):
        return get_permission_for_model(self.queryset.model, "view")

    def get(self, request, **kwargs):

        cves = get_cves(request)
        print(cves)
        table = tables.CveTable(cves)

        return render(
            request,
            self.template_name,
            {
                "tab": self.tab,
                "cves": cves,
                "filter_form": self.filterset_form,
                "table": table,
            },
        )


def get_cves(request):
    if request.GET.get("q") is not None:
        query = request.GET.get("q")
    elif request.GET.get("device_type") is not None:
        device_type_id = request.GET.get("device_type")
        device_type = DeviceType.objects.get(id=device_type_id)
        manufactor = device_type.manufacturer.name
        product = device_type.model
        version = request.GET.get("version")
        if version is None:
            version = "-"
        if request.GET.get("part") is not None:
            part = request.GET.get("part")
        else:
            part = "h"
        query = f"cpe:2.3:{part}:{manufactor}:{product}:{version}:*:*:*:*:*:*:*"
    else:
        return []

    payload = {"cpeName": f"{query}"}
    try:
        r = requests.get(
            "https://services.nvd.nist.gov/rest/json/cves/2.0/", params=payload
        )
        r.raise_for_status()
        output = []
        for entry in r.json()["vulnerabilities"]:
            cve = {"id": entry["cve"]["id"]}
            for description in entry["cve"]["descriptions"]:
                if description["lang"] == "en":
                    cve["description"] = description["value"]
                    break
            for metric in entry["cve"]["metrics"]:
                if metric == "cvssMetricV2":
                    metrics = entry["cve"]["metrics"][metric][0]["cvssData"]
                    attributes = ['accessVector', 'accessComplexity', 'authentication', 'confidentialityImpact', 'integrityImpact', 'availabilityImpact', 'baseScore']
                    for attribute in attributes:
                        if attribute in metrics:
                            cve[attribute] = metrics[attribute]
                        else:
                            cve[attribute] = ""
            output.append(cve)
        return output
    except Exception as e:
        print(e)
        return []



from extras.plugins import PluginTemplateExtension
from extras.plugins.utils import get_plugin_config
from django.conf import settings
from packaging import version

NETBOX_CURRENT_VERSION = version.parse(settings.VERSION)


def create_button(model_name):
    class Button(PluginTemplateExtension):
        model = model_name

        def buttons(self):
            return self.render("nb_risk/vulnerability_assignment_button.html")

    return Button


supported_assets = get_plugin_config("nb_risk", "supported_assets")
additional_assets = get_plugin_config("nb_risk", "additional_assets")
supported_assets = supported_assets + additional_assets

template_extensions = []
for supported_asset in supported_assets:
    template_extensions.append(create_button(supported_asset))



# Generated by Django 4.1.5 on 2023-04-21 00:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('nb_risk', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='vulnerability',
            name='cvssaccessComplexity',
            field=models.CharField(blank=True, max_length=100),
        ),
        migrations.AddField(
            model_name='vulnerability',
            name='cvssaccessVector',
            field=models.CharField(blank=True, max_length=100),
        ),
        migrations.AddField(
            model_name='vulnerability',
            name='cvssauthentication',
            field=models.CharField(blank=True, max_length=100),
        ),
        migrations.AddField(
            model_name='vulnerability',
            name='cvssavailabilityImpact',
            field=models.CharField(blank=True, max_length=100),
        ),
        migrations.AddField(
            model_name='vulnerability',
            name='cvssbaseScore',
            field=models.FloatField(blank=True, default=0, max_length=100),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='vulnerability',
            name='cvssconfidentialityImpact',
            field=models.CharField(blank=True, max_length=100),
        ),
        migrations.AddField(
            model_name='vulnerability',
            name='cvssintegrityImpact',
            field=models.CharField(blank=True, max_length=100),
        ),
    ]
